#! /usr/bin/env python

# Copyright (C) 2005-2011 Canonical Ltd
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

"""Bazaar -- a free distributed version-control tool"""

import os
import sys
import warnings

# update this on each release
_script_version = (2, 5, 0)

try:
    version_info = sys.version_info
except AttributeError:
    version_info = 1, 5  # 1.5 or older

REINVOKE = "__BZR_REINVOKE"
NEED_VERS = (2, 6)
KNOWN_PYTHONS = ('python2.6', 'python2.7')

if version_info < NEED_VERS:
    if not os.environ.has_key(REINVOKE):
        # mutating os.environ doesn't work in old Pythons
        os.putenv(REINVOKE, "1")
        for python in KNOWN_PYTHONS:
            try:
                os.execvp(python, [python] + sys.argv)
            except OSError:
                pass
    sys.stderr.write("bzr: error: cannot find a suitable python interpreter\n")
    sys.stderr.write("  (need %d.%d or later)\n" % NEED_VERS)
    sys.exit(1)
if hasattr(os, "unsetenv"):
    os.unsetenv(REINVOKE)


profiling = False
if '--profile-imports' in sys.argv:
    import profile_imports
    profile_imports.install()
    profiling = True

if sys.platform == 'darwin':
    # jameinel says this hack is to force python to honor the LANG setting,
    # even on Darwin.  Otherwise it is apparently hardcoded to Mac-Roman,
    # which is incorrect for the normal Terminal.app which wants UTF-8.
    #
    # "It might be that I should be setting the "system locale" somewhere else
    # on the system, rather than setting LANG=en_US.UTF-8 in .bashrc.
    # Switching to 'posix' and setting LANG worked for me."
    #
    # So we can remove this if someone works out the right way to tell Mac
    # Python which encoding to use.  -- mbp 20080703
    sys.platform = 'posix'
    try:
        import locale
    finally:
        sys.platform = 'darwin'
else:
    import locale

if os.name == "posix":
    try:
        locale.setlocale(locale.LC_ALL, '')
    except locale.Error, e:
        sys.stderr.write('bzr: warning: %s\n'
            '  bzr could not set the application locale.\n'
            '  Although this should be no problem for bzr itself, it might\n'
            '  cause problems with some plugins. To investigate the issue,\n'
            '  look at the output of the locale(1p) tool.\n' % e)
    # Use better default than ascii with posix filesystems that deal in bytes
    # natively even when the C locale or no locale at all is given. Note that
    # we need an immortal string for the hack, hence the lack of a hyphen.
    sys._bzr_default_fs_enc = "utf8"


# The python2.6 release includes some libraries that have deprecation warnings
# against the interpreter - see https://bugs.launchpad.net/bzr/+bug/387139
warnings.filterwarnings('ignore',
    r"(struct integer overflow masking is deprecated|"
    r"'L' format requires 0 <= number <= 4294967295)",
    DeprecationWarning,
    'gzip',
    )


# instruct bzrlib/__init__.py to install lazy_regex
sys._bzr_lazy_regex = True
try:
    import bzrlib
except ImportError, e:
    sys.stderr.write("bzr: ERROR: "
        "Couldn't import bzrlib and dependencies.\n"
        "Please check the directory containing bzrlib is on your PYTHONPATH.\n"
        "\n")
    raise

if bzrlib.version_info[:3] != _script_version:
    sys.stderr.write(
        "bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
        "This may indicate an installation problem.\n"
        "bzrlib is version %s from %s\n"
        "bzr is version %s from %s\n" % (
        bzrlib._format_version_tuple(bzrlib.version_info),
        bzrlib.__path__[0],
        bzrlib._format_version_tuple(_script_version),
        __file__))


import bzrlib.inspect_for_copy
bzrlib.inspect_for_copy.import_copy_with_hacked_inspect()

import bzrlib.breakin
bzrlib.breakin.hook_debugger_to_signal()

import bzrlib.decorators
if ('--lsprof' in sys.argv
    or '--lsprof-file' in sys.argv
    or '--profile' in sys.argv
    or '--lsprof-timed' in sys.argv):
    bzrlib.decorators.use_pretty_decorators()
else:
    bzrlib.decorators.use_fast_decorators()

import bzrlib.commands
import bzrlib.trace


if __name__ == '__main__':
    library_state = bzrlib.initialize()
    library_state.__enter__()
    try:
        exit_val = bzrlib.commands.main()
        if profiling:
            profile_imports.log_stack_info(sys.stderr)
    finally:
        library_state.__exit__(None, None, None)

    # By this point we really have completed everything we want to do, and
    # there's no point doing any additional cleanup.  Abruptly exiting here
    # stops any background threads getting into trouble as code is unloaded,
    # and it may also be slightly faster, through avoiding gc of objects that
    # are just about to be discarded anyhow.  This does mean that atexit hooks
    # won't run but we don't use them.  Also file buffers won't be flushed,
    # but our policy is to always close files from a finally block. -- mbp 20070215
    sys.exitfunc()
    os._exit(exit_val)
else:
    raise ImportError("The bzr script cannot be imported.")
